% Compute the element load vectors corresponding to prescribed pressure
% flux. (Pressure flux boundary condition is stated as
%      rho*a_n=-(d p/d n)
% where a_n= prescribed acceleration, n=direction along the outer normal
%
% function evs = pressure_flux_loads (self, geom, P, fi)
%
% Return an array of the element loads so they may be assembled.
%     
% Arguments
%     geom=geometry field
%     P=pressure field
%     fi=magnitude of the flux (force intensity)
%
function evs = pressure_flux_loads (self, geom, P, fi)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells(1),'nfens');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Precompute basis f. values + basis f. gradients wrt parametric coor
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Prepare some data: connectivity and the element matrices
    conns = get(gcells, 'conn'); % connectivity
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(P,conns(i,:),'eqnums');
        Fe{i} =zeros(nfens,1);
    end
    xs =get(geom,'values');% Retrieve Geometry information
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        for j=1:npts_per_gcell
            J = Jacobian_matrix(gcells,Nders{j},x);
            Jac = Jacobian_surface(gcells,conn, Ns{j}, J, x);
            f=get_magn(fi, Ns{j}'*x, J); 
            Fe{i} = Fe{i}  + (Jac*w(j)*f)*Ns{j};
        end
    end
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end
