% Constructor of the finite element block class.
%    This class implements small displacement, small strain deformation
%    mechanics for Nodally-Integrated Continuum Elements (NICE).
%
% The approximation is  originally from Dohrmann et al IJNME 47 (2000).
%
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing options.
% Options: those recognized by feblock
% and
%   stabfact= stabilization factor, 0 = no stabilization.
%
function retobj = feblock_defor_ss_nice (varargin)
    class_name='feblock_defor_ss_nice';
    if (nargin == 0)
        parent = feblock;
        self.stabfact=0;
        self.irrot_stabfact=0;
        self.matstates= [];
        self.hadjugate=[];
        self.hBlmat=[];
        self.hdivmat=[];
        self.hvgradmat=[];
        self.hPhi=[];
        retobj = class(self,class_name,parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            parent = feblock(varargin{:});
            options =varargin{1};
            stabfact= 0;
            if isfield(options,'stabfact')
                stabfact= options.stabfact;
            end
            self.stabfact=stabfact;
            self.irrot_stabfact=0;
            if isfield(options,'irrot_stabfact')
                self.irrot_stabfact= options.irrot_stabfact;
            end
            self.matstates= [];
            self.hadjugate=[];
            self.hBlmat=[];
            self.hdivmat=[];
            self.hvgradmat=[];
            self.hPhi=[];
            self = class(self,class_name,parent);
            gcells = get(self,'gcells');
            dim= get(gcells,'dim');
            switch dim
                case 1
                    self.hBlmat=@feblock_defor_ss_nice_Blmat1;
                    self.hdivmat=@feblock_defor_ss_nice_divmat1;
                case 2
                    self.hadjugate=@feblock_defor_ss_nice_adjugate2;
                    if get(gcells(1),'axisymm')
                        self.hBlmat=@feblock_defor_ss_nice_Blmat2axisymm;
                        self.hdivmat=@feblock_defor_ss_nice_divmat2axisymm;
                    else
                        self.hBlmat=@feblock_defor_ss_nice_Blmat2;
                        self.hdivmat=@feblock_defor_ss_nice_divmat2;
                        self.hvgradmat=@feblock_defor_ss_nice_vgradmat2;
                    end
                    self.hPhi= @feblock_defor_ss_nice_Phi2;
                case 3
                    self.hadjugate=@feblock_defor_ss_nice_adjugate3;
                    self.hBlmat=@feblock_defor_ss_nice_Blmat3;
                    self.hdivmat=@feblock_defor_ss_nice_divmat3;
                    self.hvgradmat=@feblock_defor_ss_nice_vgradmat3;
                    self.hPhi= @feblock_defor_ss_nice_Phi3;
                otherwise
                    error (' Not implemented');
            end
            f2gmap=fenode_to_gcell_map (struct ('gcells',gcells));
            gmap=get (f2gmap,'gcell_map');
            mater =get(self.feblock,'mater');
            self.matstates = cell(length(gmap),1);
            if (~isempty(mater))
                for i=1:size(self.matstates,1)
                    self.matstates{i,1} = newmatstate(mater);
                end
            end
            retobj = self;
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end
