% Inspect the integration point quantities.
% by which the individual gcells act on the nodes.
% Return an array of elevec objects so they may be assembled.
% Input arguments
%     x        - reference geometry field
%     u        - displacement field
%     dT       - temperature difference field
%     output   - this is what you would assign to the output field all of
%                the context argument of the material update() method.
%     component- component of the output array: see the material update()
%                method.
% Output argument
%     fld - the new field that can be used to map values the colors
%
function fld = field_from_integration_points(self, geom, u, dT, output, component)
    gcells = get(self.feblock,'gcells');
    dim = get(geom,'dim');
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    tangents =eye(dim);
    % Material
    mat = get(self.feblock, 'mater');
    D = tangent_moduli(mat,struct([]));
    [Princd,W]=eig(D);
    [ignore,ix]=sort(diag(W));
    Wmean=(W(ix(1),ix(1))+W(ix(2),ix(2)))/2;
    if Wmean<=0
        Wmean=max(diag(W));
    end
    %     Evaluate the nodal basis function gradients
    bfun_gradients = nodal_bfun_gradients (self, geom);
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    xs =gather(geom, 1:length(bfun_gradients), 'values', 'noreshape'); % coordinates of all nodes
    % Now loop over all finite element nodes
    fld =0*slice(u,1,'fld');
    nvals = gather (fld, (1: get(fld,'nfens')),'values');
    context.output = output;
    for nix=1:length(bfun_gradients)
        if bfun_gradients{nix}.Vpatch>0 % This node has a element patch in this block
            np=length(bfun_gradients{nix}.patchconn);
            c = xs(nix, :); % coordinates of central node
            D = tangent_moduli (mat, struct ('xyz',c));% material tangent
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,tangents);%RAW: tangent vectors?
            end
            Bnodal= self.hBlmat (self,bfun_gradients{nix}.Nspd,c,Rm);
            U = gather(u, bfun_gradients{nix}.patchconn, 'values'); % displacement
            context.strain = Bnodal*U ;
            if isempty(dT)
                dTs=zeros(length(bfun_gradients{nix}.patchconn),1);
            else
                dTs = gather(context.dT, bfun_gradients{nix}.patchconn, 'values', 'noreshape'); % displacement
            end
            context.dT = dTs(end); % temperature at the node
            context.xyz =c;
            [out,ignore] = update(mat, [], context);
            nvals (nix) = out(component);
        end
    end
    fld = scatter(fld, (1: get(fld,'nfens')),nvals);
end
