% Constructor of properties for linearly elastic isotropic materials..
%
% function retobj = property_linel_iso(varargin)
%
% This class represents material properties of deformable linearly elastic isotropic materials.
% See discussion of constructors in <a href="matlab:helpwin 'FAESOR/classes/Contents'">classes/Contents</a>. 
% Parameters:
% required fields
%     E=Young's modulus
%     nu=Poisson ratio
%     Optionally one may supply the shear modulus
%     G=shear modulus
% optional fields
%     alpha = coefficient of thermal expansion, default 0.0
%
function retobj = property_linel_iso(varargin)
    class_name='property_linel_iso';
    if (nargin == 0)
        parent=property;
        self.E = 1.0;
        self.nu = 0.0;
        self.G = [];% the default is to compute the shear modulus from the two above
        self.D =[];
        self.alpha = 0.0;
        retobj = class(self,class_name, parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            Parameters =varargin{1};
            parent=property(Parameters);
            self.E = Parameters.E;
            self.nu = 0;
            self.G = [];% the default is to compute the shear modulus from the two above
            self.D =[];
            if isfield(Parameters,'nu')
                self.nu = Parameters.nu;
                if (self.nu < 0)
                    error('Negative Poisson ratio!');
                elseif (self.nu >= 0.5)
                    error('Incompressible material!');
                end
            end
            if isfield(Parameters,'G')
                self.G = Parameters.G;
                if (self.G < 0)
                    error('Negative shear modulus!');
                end
                self.nu = [];
            end
            self.alpha = 0.0;
            if isfield(Parameters,'alpha')
                self.alpha= Parameters.alpha;
            end
            % precompute
            E = self.E;
            nu = self.nu;
            if (isempty(nu))
                nu =E/self.G/2-1;
                if (self.nu < 0)
                    error('Negative Poisson ratio!');
                elseif (self.nu >= 0.5)
                    error('Incompressible material!');
                end
            end
            lambda = E * nu / (1 + nu) / (1 - 2*(nu));
            mu     = E / (2 * (1 + nu));
            mI = diag([1 1 1 0.5 0.5 0.5]);
            m1 = [1 1 1 0 0 0]';
            self.D = lambda * m1 * m1' + 2 * mu * mI;
            retobj = class(self,class_name, parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end
