%%  Argyris L-frame, buckling, corotational beam

%% 
% References
% J.H. Argyris, P.C. Dunne and D.W. Scharpf, On large 
% displacement-small strain analysis of structures with 
% rotation degree of freedom. Comput. Methods Appl. Mech. Engrg. 14 (1978) 401-451; 15 (1978) 99-135. 
% J.H. Argyris, 0. Hilpert, GA. Malejannakis and D.W. Scharpf, On the 
% geometrical stiffness of a beam in space-A consistent V.W. 
% Approach, Comput. Methods Appl. Mech. Engrg. 20 (1979) 105-131. 
% 
% The reference values of the buckling parameters are 68040 (for the load
% putting the clamped leg into compression) and -108800 (for the load
% putting the clamped leg in tension).

%% 
% Parameters:
% The parameters used below are physical.
E=71240.0;%MPa
nu=0.31;% Poisson ratio
b=0.6; h=30; L=240;% cross-sectional dimensions and length of each leg in millimeters
magn=-1e-5;% Magnitude of the total applied force, Newton


neigvs=2;% Number of eigenvalues to solve for


% Mesh

%% 
% Mesh
n=3;
tolerance=L/n/100;
Parameters=struct ('A',b*h,'I2',b*h^3/12,'I3',b^3*h/12,'IT',b^3*h/3,...
    'x1x2_vector',[0 1 0], 'label',1);
[fens1,gcells1] = Beam_member([0,0,L;L,0,L],n,@gcellset_beam3,Parameters);
[fens2,gcells2] = Beam_member([L,0,L;L,0,0],n,@gcellset_beam3,Parameters);
[fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens2, gcells2, tolerance);
gcells= cat(gcells1,gcells2);

% The clamped end is
clampedn=fenode_select(fens, struct('box', update_box([],[0,0,L]), 'inflate', tolerance));
% The force is applied at the tip
tipn=fenode_select(fens, struct('box', update_box([],[L,0,0]), 'inflate', tolerance));

%% 
% Properties
prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));

%% 
% Finite element block. Nonlinear co-rotational beam element is used.
feb = feblock_defor_ss_beam3_corot (struct ('mater',mater, 'gcells',gcells));

%% 
% Geometry and displacement fields
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
ur  = field(struct ('name',['ur'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));
Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom,'nfens'),1)*reshape(eye(3),1,9)));

%% 
% Apply EBC's
ebc_fenids=[clampedn];
ebc_prescribed=[1];
ebc_comp=[];
ebc_val=ebc_fenids*0;
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ur  = apply_ebc (ur);


%% 
% Number equations
ur  = numbereqns (ur);
neqns=get(ur, 'neqns');


%% 
% Assemble and solve the discrete system for static displacement
K = start (sparse_sysmat, neqns);
K = assemble (K, stiffness(feb, geom, ur, Rfield));
% Load
F = sysvec;
F = start (F, neqns);
evs = loads(nodal_load(struct ('id',tipn,'dir',[1],'magn',[magn])), ur);
F = assemble (F, evs);
% Solve 
ur = scatter_sysvec(ur, K\F);
load_displacement=somel(gather(ur,tipn,'values','noreshape'),1:3)



%% 
% Display the deformed structure
gv=graphic_viewer;
gv=reset (gv,[]);
scale=20000000;
u=slice(ur,(1:3),'u');
rot=slice(ur,(4:6),'rot');
draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
draw(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', scale*rot, 'facecolor','none'));
draw_local_coordinates(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'length',25));

xlabel('X');
ylabel('Y');
zlabel('Z');
hold on

draw(fens,gv, struct ('x', geom, 'u', 0*u, 'color','blue','offset',0.35));
draw_axes(gv, struct([]));


%% 
% Proceed to set up the geometric stiffness
Kg = start (sparse_sysmat, neqns);
Kg = assemble (Kg, stiffness_geo(feb, geom, ur, update_rotation_field(Rfield,slice(ur,(4:6),'rot'))));

%% 
% Solve the buckling problem
options.tol =1e-26;
options.maxit = 5000;
options.disp = 0;
[W,Omega]=eigs(-(get(Kg,'mat')+get(Kg,'mat')')/2,(get(K,'mat')+get(K,'mat')')/2,...
    neigvs,'lm', options);
[Omegas,ix]=sort(1./abs(diag(Omega))) ;
Omegas=somel(1./diag(Omega),ix);

%
for i=1:neigvs
    disp(['  Eigenvector ' num2str(i) ' eigenvalue ' num2str(Omegas(i)) ]);
end
disp(' ')


%% 
% Graphic display of the buckling modes
scale=15;
w=clone(ur,'w'); % make a copy of u
for i=1:neigvs
    ur = scatter_sysvec(w, W(:,i));
    u=slice(ur,(1:3),'u');
    rot=slice(ur,(4:6),'rot');
    wv = get(u,'values');
    wmag = sqrt(wv(:,1).^2+wv(:,2).^2+wv(:,3).^2);
    dcm=data_colormap(struct ('range',[min(wmag),max(wmag)], 'colormap',jet));
    colors=map_data(dcm, wmag);
    colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
    gv=graphic_viewer;
    gv=reset (gv,[]);
    gv=reset (gv,[]);
    draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
    draw(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', scale*rot,'colorfield',colorfield));
    draw_axes(gv, struct([]));
    title(['  Eigenvector ' num2str(i) ' eigenvalue ' num2str(Omegas(i)) ]);
    pause(2); % next eigenvector
end


