%%  Cantilever beam, Tippling under transverse end force. Corotational beam

%% 
% The beam counterpart of CTippling. The continuum elements give 
% the buckling parameters for the first two modes 54.2759, 145.0105.


%% 
% Since the deflection under the load is substantial, the nonlinear
% solution for the buckling loads gives different values for positive and
% negative direction. When the magnitude of the load is adjusted to a
% fraction that guarantees very small deflections, the buckling factors
% come out close together.

%% 
% Parameters
E=1000000;
nu=0.3;
L=  30; % Length of the beam
b= 0.5; % width
h= 4; % height
magn=-0.2056167583560*1e4/L^2;

neigvs = 4;

% Mesh
n=8;%number of elements
tolerance=L/n/100;
Parameters=struct ('A',b*h,'I2',b*h^3/12,'I3',b^3*h/12,'IT',b^3*h/3,...
        'x1x2_vector',[-1 0 0], 'label',1);
[fens,gcells] = Beam_member([0,0,0;0,0,L],n,@gcellset_beam3,Parameters);

% The clamped end 
clampedn=fenode_select(fens, struct('box', update_box([],[0,0,0]), 'inflate', tolerance));
% The tip end
tipn=fenode_select(fens, struct('box', update_box([],[0,0,L]), 'inflate', tolerance));

prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));
% Finite element block
feb = feblock_defor_ss_beam3_corot (struct ('mater',mater, 'gcells',gcells));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
ur  = field(struct ('name',['ur'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));
Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom,'nfens'),1)*reshape(eye(3),1,9)));

% Apply EBC's
ebc_fenids=[clampedn];
ebc_prescribed=[1];
ebc_comp=[];
ebc_val=0;
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ur  = apply_ebc (ur);

% Number equations
ur  = numbereqns (ur);
neqns=get(ur, 'neqns');

% Assemble the system matrix
K = start (dense_sysmat, neqns);
K = assemble (K, stiffness(feb, geom, ur, Rfield));
% Load
F = sysvec;
F = start (F, neqns);
evs = loads(nodal_load(struct ('id',tipn,'dir',[2],'magn',[-magn*b*h])), ur);
F = assemble (F, evs);
% Solve
ur = scatter_sysvec(ur, K\F);
tip_displacement= gather(ur,tipn,'values')


% Plot
gv=graphic_viewer;
gv=reset (gv,[]);
scale=100;
u=slice(ur,(1:3),'u');
rot=slice(ur,(4:6),'rot');
draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
draw(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', scale*rot, 'facecolor','red'));

xlabel('X');
ylabel('Y');
zlabel('Z');
hold on

draw(fens,gv, struct ('x', geom, 'u', 0*u, 'color','blue','offset',0.35));
draw_axes(gv, struct([]));

Kg = start (dense_sysmat, neqns);
Kg = assemble (Kg, stiffness_geo(feb, geom, ur, update_rotation_field(Rfield,slice(ur,(4:6),'rot'))));

options.tol =1e-26;
options.maxit = 5000;
options.disp = 0;
[W,Omega]=eigs(-(get(Kg,'mat')+get(Kg,'mat')')/2,(get(K,'mat')+get(K,'mat')')/2,...
    neigvs,'lm', options);
diag(Omega)
[Omegas,ix]=sort(1./abs(diag(Omega))) ;
Omegas=somel(1./diag(Omega),ix);
  
  
%
for i=1:neigvs
    disp(['  Eigenvector ' num2str(i) ' eigenvalue ' num2str(Omegas(i)) ]);
end
disp(' ')

gv=graphic_viewer;
gv=reset (gv,[]);
scale=50;
w=clone(ur,'w'); % make a copy of u
for i=1:4
    ur = scatter_sysvec(w, W(:,ix(i)));
    u=slice(ur,(1:3),'u');
    rot=slice(ur,(4:6),'rot');
    wv = get(u,'values');
    wmag = sqrt(wv(:,1).^2+wv(:,2).^2+wv(:,3).^2);
    dcm=data_colormap(struct ('range',[min(wmag),max(wmag)], 'colormap',jet));
    colors=map_data(dcm, wmag);
    colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
    gv=reset (gv,[]);
    draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
    draw(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', scale*rot,'colorfield',colorfield));
    draw_axes(gv, struct([]));
    title(['  Eigenvector ' num2str(i) ' eigenvalue ' num2str(Omegas(i)) ]);
    pause(2); % next eigenvector
end


