%%  Argyris L-frame, out-of-plane load, nonlinear.


%% 
% Parameters:
E=71240.0;%MPa
nu=0.31;% Poisson ratio
b=3; h=30; L=240;% cross-sectional dimensions and length of each leg in millimeters
magn=40;% Magnitude of the total applied force, Newton
utol=1e-3;

%% 
% Mesh
n=8;
tolerance=L/n/100;
Parameters=struct ('A',b*h,'I2',b*h^3/12,'I3',b^3*h/12,'IT',b^3*h/3,...
    'x1x2_vector',[0 1 0], 'label',1);
[fens1,gcells1] = Beam_member([0,0,L;L,0,L],n,@gcellset_beam3,Parameters);
[fens2,gcells2] = Beam_member([L,0,L;L,0,0],n,@gcellset_beam3,Parameters);
[fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens2, gcells2, tolerance);
gcells= cat(gcells1,gcells2);

% The clamped end is
clampedn=fenode_select(fens, struct('box', update_box([],[0,0,L]), 'inflate', tolerance));
% The force is applied at the tip
tipn=fenode_select(fens, struct('box', update_box([],[L,0,0]), 'inflate', tolerance));

%% 
% Properties
prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));

%% 
% Finite element block
feb = feblock_defor_ss_beam3_corot(struct ('mater',mater, 'gcells',gcells));

%% 
% Geometry and displacement fields
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
ur  = field(struct ('name',['ur'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));
Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom,'nfens'),1)*reshape(eye(3),1,9)));

%% 
% Apply EBC's
ebc_fenids=[clampedn];
ebc_prescribed=[1];
ebc_comp=[];
ebc_val=ebc_fenids*0;
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ur  = apply_ebc (ur);


%% 
% Number equations
ur  = numbereqns (ur);
neqns=get(ur, 'neqns');

gv=graphic_viewer;
    

load_parameters=linspace(0, 1, 5);
crown_displacement=[];
for load_parameter=load_parameters
    
    % Initialization
    ur1 = ur; % guess
    ur1 = apply_ebc(ur1);
    Rfield1=Rfield;
    dur = 0*ur; % this will hold displacement increment
    dur = apply_ebc(dur);
    nl=nodal_load(struct ('id',tipn,'dir',[2],'magn',[-load_parameter*magn]));
    iter=1;
    while 1
        feb1=feb;  % Make a copy of the state
        F = start (sysvec, neqns);
        F = assemble (F, loads(nl, ur1));
        [feb1,evs] = restoring_force(feb1,geom,ur1,Rfield1);       % Internal forces
        F = assemble (F, evs);
        K = start (sparse_sysmat, neqns);
        K = assemble (K, cat(2,stiffness(feb1, geom,ur1,Rfield1),stiffness_geo(feb1, geom,ur1,Rfield1)));
        dur = scatter_sysvec(dur, K\F); % Displacement increment
        % Line search
        R0 = dot(get(F,'vec'),gather_sysvec(dur));
        F = start (sysvec, neqns);
        F = assemble (F, loads(nl, ur1+dur));
        [feb1,evs] = restoring_force(feb1,geom,ur1+dur,update_rotation_field(Rfield1,slice(dur,(4:6),'rot')));       % Internal forces
        F = assemble (F, evs);
        R1 = dot(get(F,'vec'),gather_sysvec(dur));
        a = R0/R1;
        if ( a<0 )
            eta = a/2 +sqrt((a/2)^2 -a);
        else
            eta =a/2;
        end
        eta=min( [eta, 1.0] );
        ur1 = ur1 + eta*dur;   % increment displacement
        Rfield1=update_rotation_field(Rfield1,eta*slice(dur,(4:6),'rot'));
        disp(['   It. ' num2str(iter) ': ||du||=' num2str(norm(dur))]);
        if (max(abs(gather(dur,[],'values'))) < utol) break; end;                    % convergence check
        iter=iter+1;
    end
    [feb,ignore] = restoring_force(feb,geom,ur1,Rfield1);        % final update
    disp(['    Converged for load_parameter=' num2str(load_parameter)]); % pause
    ur = ur1; % update the displacement
    Rfield=Rfield1; % update the rotations
    utip=gather(ur,get(nl, 'id'),'values');
    disp(['Crown deflection =' num2str(utip(3),'%27.15e\n')])
    crown_displacement=[crown_displacement,utip(3)];
    
    %%
    % Graphic display of the mesh and the deformed beam
    gv=reset (gv,[]);
    scale=1;
    u=slice(ur1,(1:3),'u');
    rot=slice(ur,(4:6),'rot');
    draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
    draw(feb, gv, struct ('x', geom, 'u', u, 'R', Rfield1, 'facecolor','m'));
%     draw_local_coordinates(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', 0*rot, 'length',25));
    
    xlabel('X');
    ylabel('Y');
    zlabel('Z');
    hold on
    
    draw(fens,gv, struct ('x', geom, 'u', 0*u, 'color','blue','offset',0.35));
    draw_axes(gv, struct([]));
%     camset (gv, [-224.6266 -292.7391  216.4415         0         0    3.4053         0         0    1.0000    4.3101]);
    pause(0.1)
end

