%%  Curved (45 deg circular arc) cantilever, Transverse force at the tip


%% 
% Large-deflection problem. Solved many times:
% 
% Bathe, Boulourchi 1979
% Simo, Vu-Quoc 1986
% Cardona, Geradin 1988
% Krysl 1993
% Krysl, FAESOR script curved_cantilever

% Present calculation refers to the data of
% Reference: EULERIAN FORMULATION FOR LARGE-DISPLACEMENT ANALYSIS OF SPACE
% FRAMES, by B.  A.  lzzuddin I and A.  S.  Elnashai,  
% Journal of Engineering Mechanics, Vol.  119, No.  3,  March, 1993. 

%% 
%
% Parameters:
E=1e7;% lb.in^-2
nu=0.3;
h= 1; b =1;% in
Fmag=600;%  lb
radius=100;% in
ang=45;
nel=8;% number of elements
utol=1e-6;

% Mesh
xyz=[];
id=[];
for i=0:nel
    id(end+1)=i+1;
    xyz=[xyz;[radius*cos(i/nel*ang/360*2*pi) radius*sin(i/nel*ang/360*2*pi) 0]];
end
fens=fenodeset(struct ('id',id,'xyz',xyz));
tipn=nel+1;

id=[];
conns = [];
for i=1:nel
    id(end+1)=i; conns= [conns;[i i+1]];
end
gcells = gcellset_beam3(struct ('id',id,'conn',conns, ...
        'A',b*h,'I2',b*h^3/12,'I3',b^3*h/12,'IT',0.281*h^3*b^3/(h^2+b^2),...
        'x1x2_vector',[0,0,1]));
    

% Material
prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));
% Finite element block
feb = feblock_defor_ss_beam3_corot(struct ('mater',mater, 'gcells',gcells));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
ur  = field(struct ('name',['ur'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));
Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom,'nfens'),1)*reshape(eye(3),1,9)));

% Apply EBC's
ebc_fenids=[1];
ebc_prescribed=[1];
ebc_comp=[];
ebc_val=ebc_fenids*0;
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ur  = apply_ebc (ur);
% Number equations
ur  = numbereqns (ur);
neqns=get(ur, 'neqns');

gv=graphic_viewer;
    

load_parameters=0.05:0.05:1;
tip_displacement=[];
for load_parameter=load_parameters
    
    % Initialization
    ur1 = ur; % guess
    ur1 = apply_ebc(ur1);
    Rfield1=Rfield;
    dur = 0*ur; % this will hold displacement increment
    dur = apply_ebc(dur);
    nl = nodal_load(struct ('id',tipn,'dir',[3],'magn',[load_parameter*Fmag]));

    iter=1;
    while 1
        feb1=feb;  % Make a copy of the state
        F = start (sysvec, neqns);
        F = assemble (F, loads(nl, ur1));
        [feb1,evs] = restoring_force(feb1,geom,ur1,Rfield1);       % Internal forces
        F = assemble (F, evs);
        K = start (sparse_sysmat, neqns);
        K = assemble (K, cat(2,stiffness(feb1, geom,ur1,Rfield1),stiffness_geo(feb1, geom,ur1,Rfield1)));
       dur = scatter_sysvec(dur, K\F); % Displacement increment
        % Line search
        R0 = dot(get(F,'vec'),gather_sysvec(dur));
        F = start (sysvec, neqns);
        F = assemble (F, loads(nl, ur1+dur));
        [feb1,evs] = restoring_force(feb1,geom,ur1+dur,update_rotation_field(Rfield1,slice(dur,(4:6),'rot')));       % Internal forces
        F = assemble (F, evs);
        R1 = dot(get(F,'vec'),gather_sysvec(dur));
        a = R0/R1;
        if ( a<0 )
            eta = a/2 +sqrt((a/2)^2 -a);
        else
            eta =a/2;
        end
        eta=min( [eta, 1.0] );
        ur1 = ur1 + eta*dur;   % increment displacement
        Rfield1=update_rotation_field(Rfield1,eta*slice(dur,(4:6),'rot'));
        disp(['   It. ' num2str(iter) ': ||du||=' num2str(norm(dur))]);
        if (max(abs(gather(dur,[],'values'))) < utol) break; end;                    % convergence check
        iter=iter+1;
    end
    [feb,ignore] = restoring_force(feb,geom,ur1,Rfield1);        % final update
    disp(['    Converged for load_parameter=' num2str(load_parameter)]); % pause
    ur = ur1;                                               % update the displacement
    Rfield=Rfield1; % update the rotations
    utip=gather(ur,get(nl, 'id'),'values');
    disp(['Tip deflection =' num2str(utip(1:3)','%27.15e, %27.15e, %27.15e\n')])
    tip_displacement=[tip_displacement;utip(1:3)'];
    
    %%
    % Graphic display of the mesh and the deformed beam
    gv=reset (gv,[]);
    scale=1;
    u=slice(ur1,(1:3),'u');
    rot=slice(ur,(4:6),'rot');
    draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
    draw(feb, gv, struct ('x', geom, 'u', u, 'R', Rfield1, 'facecolor','g'));
%     draw_local_coordinates(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', 0*rot, 'length',25));
    
    xlabel('X');
    ylabel('Y');
    zlabel('Z');
    hold on
    
    draw(fens,gv, struct ('x', geom, 'u', 0*u, 'color','blue','offset',0.35));
    draw_axes(gv, struct([]));
   camset (gv, [-392.3967 -581.0903  474.9767   71.8588   23.9391   34.6762 0         0    1.0000    3.4650]);
    pause(1)
end


%% 
% Comparison with reference solution.

izzelna_ux =[    0.0317    0.6048
   -0.0628    0.6048
    8.7273   34.7095
   16.6668   85.8664
   23.7557  134.8918
   32.6404  213.7587
   37.8389  264.9157
   41.7141  318.2041
   44.9277  382.1503
   47.9523  446.0965
   50.4098  510.0426
   51.9220  578.2519
   53.9069  657.1188
   53.9069  657.1188]

izzelna_uy =[-0.1574    0.6048
    0.0317    2.7364
    1.1659   73.0772
    2.7727  132.7603
    4.9466  211.6272
    7.1205  284.0995
    8.9164  350.1772
   10.2396  414.1234
   11.3738  480.2011
   13.1697  548.4104
   14.0203  618.7511
   14.6820  657.1188
   14.5875  657.1188];




izzelna_uz =[   -0.5354    2.7364
   -0.2519    2.7364
   -1.1970   51.7618
   -2.2367   90.1295
   -4.3161  124.2341
   -6.8681  183.9172
   -9.1366  235.0741
  -11.8776  288.3626
  -14.5241  354.4403
  -16.9815  411.9919
  -19.3445  475.9380
  -21.7074  550.5419
  -23.9759  618.7511
  -25.3936  661.3819];


figure
plot(izzelna_ux(:,1),izzelna_ux(:,2),'r-','linewidth',3)
hold on
plot(izzelna_uy(:,1),izzelna_uy(:,2),'r-','linewidth',3)
hold on
plot(izzelna_uz(:,1),izzelna_uz(:,2),'r-','linewidth',3)
hold on
plot(tip_displacement(:,1),load_parameters*Fmag,'x','linewidth',3)
plot(tip_displacement(:,2),load_parameters*Fmag,'x','linewidth',3)
plot(tip_displacement(:,3),load_parameters*Fmag,'x','linewidth',3)
title('Curved cantilever, L-to-R uy,ux,uz','fontname','Times','fontsize',16)
labels(' Tip displacement [in]','Load [lb]')
grid on
set_graphics_defaults

