%%  Simply-supported beam with distributed load

%% 
% Parameters:
E=30002*1000;%30002ksi
b=2;%in
h=18;%in
L=240;% in
q=[1;0;0];% 1200 lbf/ft
%% 
% Cross-sectional properties
A=b*h;%in^2
I2=b*h^3/12;%cm^4 
I3=b^3*h/12;%cm^4 
nu=0.;

utol=1e-1;

%% 
% Exact deformation under the load
P_Euler=(pi^2*E*I3/L^2);
deflex=5*norm(q)*L^4/(384*E*I3);

%% 
% The problem is essentially solved exactly with a single element.
n=4;
Parameters=struct ('A',A,'I2',I2,'I3',I3,'IT',I2+I3,...
    'x1x2_vector',[1 0 0], 'label',1);
[fens,gcells] = Beam_member([[0 -L/2 0];[0 L/2 0]],2*n,@gcellset_beam3,Parameters);
midn=fenode_select(fens, struct('box', update_box([],[0 0 0])));
forcen=fenode_select(fens, struct('box', update_box([],[0 -L/2 0])));

%% 
% Material properties
prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));
%% 
% Finite element block
feb = feblock_defor_ss_beam3_corot(struct ('mater',mater, 'gcells',gcells));


%% 
% Construct the requisite fields, geometry and displacement

geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
ur  = field(struct ('name',['ur'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));
Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom,'nfens'),1)*reshape(eye(3),1,9)));

% Apply EBC's
ebc_fenids=forcen;
ebc_prescribed=[1];
ebc_comp=[1,3,5];% cylindrical, with compressive force loading 
ebc_val=ebc_fenids*0;
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ebc_fenids=fenode_select(fens, struct('box', update_box([],[0 L/2 0])));
ebc_prescribed=[1];
ebc_comp=[1,2,3,5];% cylindrical
ebc_val=ebc_fenids*0;
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ur  = apply_ebc (ur);

% Number equations
ur  = numbereqns (ur);
neqns=get(ur, 'neqns');

%% 
% Solver
fi = force_intensity(struct('magn' ,q ));

load_parameters=0.99:0.05:1.5;
mid_displacement=[];
for load_parameter=load_parameters
    
    % Initialization
    ur1 = ur; % guess
    ur1 = apply_ebc(ur1);
    Rfield1=Rfield;
    dur = 0*ur; % this will hold displacement increment
    dur = apply_ebc(dur);
    nl=nodal_load(struct ('id',forcen,'dir',[2],'magn',[load_parameter*P_Euler]));
    iter=1;
    while 1
        feb1=feb;  % Make a copy of the state
        F = start (sysvec, neqns);
        F = assemble (F, distrib_loads(feb, geom, ur1,Rfield1, fi));
        F = assemble (F, loads(nl, ur1));
        [feb1,evs] = restoring_force(feb1,geom,ur1,Rfield1);       % Internal forces
        F = assemble (F, evs);
        K = start (sparse_sysmat, neqns);
   %     K = assemble (K, cat(2,stiffness(feb1, geom,ur1,Rfield1),stiffness_geo(feb1, geom,ur1,Rfield1)));
         K = assemble (K, cat(2,stiffness(feb1, geom,ur1,Rfield1)));
        dur = scatter_sysvec(dur, K\F); % Displacement increment
         % Line search
        R0 = dot(get(F,'vec'),gather_sysvec(dur));
        F = start (sysvec, neqns);
        F = assemble (F, loads(nl, ur1+dur));
        [feb1,evs] = restoring_force(feb1,geom,ur1+dur,update_rotation_field(Rfield1,slice(dur,(4:6),'rot')));       % Internal forces
        F = assemble (F, evs);
        R1 = dot(get(F,'vec'),gather_sysvec(dur));
        a = R0/R1;
        if ( a<0 )
            eta = a/2 +sqrt((a/2)^2 -a);
        else
            eta =a/2;
        end
        eta=min( [eta, 1.0] );
        ur1 = ur1 + eta*dur;   % increment displacement
        Rfield1=update_rotation_field(Rfield1,eta*slice(dur,(4:6),'rot'));
        disp(['   It. ' num2str(iter) ': ||du||=' num2str(norm(dur))]);
        if (max(abs(gather(dur,[],'values'))) < utol) break; end;                    % convergence check
        iter=iter+1;
    end
    [feb,ignore] = restoring_force(feb,geom,ur1,Rfield1);        % final update
    disp(['    Converged for load_parameter=' num2str(load_parameter)]); % pause
    ur = ur1;                                               % update the displacement
    Rfield=Rfield1; % update the rotations
    umid=gather(ur,midn,'values');
    disp(['Point deflection =' num2str(umid(1),'%27.15e\n')])
    mid_displacement=[mid_displacement,umid(1)];
    
    %%
    % Graphic display of the mesh and the deformed beam
    gv=graphic_viewer;
    gv=reset (gv,[]);
    scale=1;
    u=slice(ur1,(1:3),'u');
    rot=slice(ur,(4:6),'rot');
    draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
    draw(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', scale*rot, 'facecolor','m'));
%     draw_local_coordinates(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', 0*rot, 'length',25));
    
    xlabel('X');
    ylabel('Y');
    zlabel('Z');
    hold on
    
    draw(fens,gv, struct ('x', geom, 'u', 0*u, 'color','blue','offset',0.35));
    draw_axes(gv, struct([]));
    camset (gv, [ -244.7965 -277.9378  215.5132  -20.1699   14.8013    2.4770         0         0    1.0000   19.1924]);
    pause(1)
end


%% 
% Comparison with reference solution.

figure
% plot(izzelna_xy(:,1),izzelna_xy(:,2),'r-','linewidth',3)
% hold on
plot(load_parameters,mid_displacement,'x','linewidth',3)
labels('Load parameter','Midpoint deflection [in]')
grid on
set_graphics_defaults