% Merge two meshes together by gluing together nodes within tolerance.
%
% function [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens2,
%                     gcells2, tolerance)
%
% The two meshes, fens1, gcells1, and fens2, gcells2, are glued together by
% merging the nodes that fall within a box of size "tolerance". If tolerance 
% is set to zero, no merging of nodes is performed; the two meshes are 
% simply concatenated together.
% 
% The merged node array, fens, and the two arrays of geometry cells with
% renumbered  connectivities are returned.
% 
% For efficiency reasons, we should make sure that
% length(fens1)<length(fens2).
% 
function [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens2, gcells2, tolerance)
    xyz1 = get(fens1,'xyz');
    dim =size(xyz1,2); 
    id1 = get(fens1,'id');
    xyz2 = get(fens2,'xyz');
    id2 = get(fens2,'id');
    c1=ones(size(xyz2,1),1);
    % Mark nodes from the first array that are duplicated in the second
    if (tolerance>0)% should we attempt to merge nodes?
        for i=1:count(fens1)
            xyz =xyz1(i,:);
            xyzd=abs(xyz2-c1*xyz);
            j=find(sum(xyzd')'<tolerance);
            if (~isempty(j))
                id1(i) =-j(1);
            end
        end
    end
    % Generate  merged arrays of the nodes
    xyzm = zeros(count(fens1)+count(fens2),dim);
    xyzm(1:count(fens2),:)=xyz2;
    idm = zeros(count(fens1)+count(fens2),1);
    idm(1:count(fens2))=(1:count(fens2));
    mid=count(fens2)+1;
    for i=1:count(fens1) % and then we pick only non-duplicated fens1
        if id1(i)>0
            id1(i)=mid;
            idm(mid)=mid;
            xyzm(mid,:)=xyz1(i,:);
            mid=mid+1;
        else 
            id1(i)=id2(-id1(i));
        end
    end
    nfens =mid-1;
    xyzm =xyzm(1:nfens,:);
    idm =idm(1:nfens,:);
    % Renumber the cells
    conn1 =get(gcells1,'conn');
    for i=1:count(gcells1)
        conn=conn1(i,:);
        conn=id1(conn);
        conn1(i,:)=conn;
    end
    gcells1=set(gcells1,'conn',conn1);
    fens =fens1;
    fens=set(fens,'xyz',xyzm);
    fens=set(fens,'id',idm);
 
end
